/*	-- $Header: P:/CAN/I386/CAN/net/RCS/copnet.c 1.1 2009/01/26 15:50:34 saturn Sav saturn $ --
 *
 *	project   :  CAN - Controller Area Network.
 *
 *	purpose   :  Interfacing CANopen with TCP/IP.
 *
 *	copyright :  (c) 2009, UV Software, Friedrichshafen.
 *
 *	compiler  :  Microsoft Visual C/C++ Compiler (Version 6.0)
 *
 *	syntax    :  copnet <address>:<port> [<options>...]
 *
 *	libraries :  ws2_32.lib
 *
 *	includes  :  dosopt.h
 *
 *	author    :  Uwe Vogt, UV Software, Friedrichshafen
 *
 *	e-mail    :  uwe.vogt@uv-software.de
 *
 *
 *	-----------  description  -----------------------------------------------
 *
 *	copnet    :  Interfacing CANopen with TCP/IP (Client).
 *
 *
 *	-----------  history  --------------------------------------------------
 *
 *	$Log: copnet.c $
 *	Revision 1.1  2009/01/26 15:50:34  saturn
 *	Initial revision
 *
 */

static const char* __prog_name    = "copnet";
static const char* __prog_date    = "26.01.2009";
static const char* __prog_version = "0.1.1";


/*	-----------  options  ---------------------------------------------------
 */

#define PROMPT_STR		0
#define PROMPT_CHR		1
#define ECHO_STR		2
#define ECHO_CHR		3
#define HELP            4
#define QUESTION_MARK   5
#define COPYRIGHT       6
#define CHARACTER_MJU   7
#define MAX_OPTIONS     8

static char* option[MAX_OPTIONS] = {
	"PROMPT", "p",
	"ECHO", "e",
	"HELP", "?",
	"ABOUT", ""
};


/*	-----------  includes  -------------------------------------------------
 */

#include "dosopt.h"						// Kommandozeilenoptionen

#include <stdio.h>						// Standard I/O routines
#include <errno.h>						// System wide error numbers
#include <string.h>						// String manipulation functions
#include <stdlib.h>						// Commonly used library functions
#include <windows.h>					// Master include-file for Windows

#include <signal.h>						// Signal values and routines
#include <winsock.h>					// Windows sockets (ws2_32.lib)


/*	-----------  defines  --------------------------------------------------
 */

#define BUFFER_LENGTH	1025
#define SEQUENCE_NO		1


/*	-----------  types  ----------------------------------------------------
 */


/*	-----------  prototypes  -----------------------------------------------
 */

static void sigterm(int signo);
static void usage(FILE *stream);
static void version(FILE *stream);

size_t readline(int fd, char *buf, size_t nbyte);


/*	-----------  variables  ------------------------------------------------
 */

static int running = 1;


/*	-----------  main  -----------------------------------------------------
 */

int main(int argc, char *argv[])
/*
 *	function  :  main function of the application.
 *
 *	parameter :  argc - number of command line arguments
 *	             argv - command line arguments as string vector
 *
 *	result    :  0    - no error occured
 */
{
    WSADATA wsaData;                 /* Structure for WinSock setup communication */

	int  i, ip = 0, echo = 0, prompt = 0;
	long ip1=127, ip2=0, ip3=0, ip4=1, port;
	long sequence = SEQUENCE_NO;
	long request, response;
	char buffer[BUFFER_LENGTH];
	struct sockaddr_in addr;
	int  client;

	// scan command-line
	while((i = getOption(argc, argv, MAX_OPTIONS, option)) != EOF) {
		switch(i) {
		case PROMPT_STR:
		case PROMPT_CHR:
			prompt = 1;
			break;
		case ECHO_STR:
		case ECHO_CHR:
			echo = 1;
			break;
		case HELP:
		case QUESTION_MARK:
			usage(stderr);
			return 0;
		case COPYRIGHT:
		case CHARACTER_MJU:
			version(stderr);
			return 0;
		default:
			fprintf(stderr, "+++ Fehler: ungltiges Argument!\n");
			return 1;
		}
	}
	for(i = 1; i < argc; i++) {
		if(!isOption(argc, argv, MAX_OPTIONS, option, i)) {
			if((ip++)) {
				fprintf(stderr, "+++ Fehler: zu viele Argumente!\n");
				return 1;
			}
			if(sscanf(argv[i], "%li.%li.%li.%li:%li", &ip1,&ip2,&ip3,&ip4, &port) == 5) {
				if((ip1 < 0) || (255 < ip1)) {
					fprintf(stderr, "+++ Fehler: ungltige Adresse!\n");
					return 1;
				}
				if((ip2 < 0) || (255 < ip2)) {
					fprintf(stderr, "+++ Fehler: ungltige Adresse!\n");
					return 1;
				}
				if((ip3 < 0) || (255 < ip3)) {
					fprintf(stderr, "+++ Fehler: ungltige Adresse!\n");
					return 1;
				}
				if((ip4 < 0) || (255 < ip4)) {
					fprintf(stderr, "+++ Fehler: ungltige Adresse!\n");
					return 1;
				}
				if((port < 0) || (65535 < port)) {
					fprintf(stderr, "+++ Fehler: ungltiger Port!\n");
					return 1;
				}
			}
			else if(sscanf(argv[i], "localhost:%li", &port) == 1) {
				if((port < 0) || (65535 < port)) {
					fprintf(stderr, "+++ Fehler: ungltiger Port!\n");
					return 1;
				}
			}
			else {
					fprintf(stderr, "+++ Fehler: ungltige Adresse und Port!\n");
					return 1;
				}
		}
	}
	if(!ip) {
		fprintf(stderr, "+++ Fehler: zu wenig Argumente!\n");
		return 1;
	}
    if (WSAStartup(MAKEWORD(2, 0), &wsaData) != 0) /* Load Winsock 2.0 DLL */
    {
        fprintf(stderr, "+++ Fataler Fehler: Windows Socket API nicht geladen!\n");
        exit(1);
    }
	signal(SIGINT, sigterm);
	signal(SIGTERM, sigterm);

	memset(&addr, 0, sizeof(addr));
	addr.sin_family = AF_INET;
	addr.sin_addr.s_addr = ip1 + ip2 * (256) + ip3 * (256*256) + ip4 * (256*256*256);
	addr.sin_port = htons((unsigned short)port);

	if((client = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
		fprintf(stderr, "+++ Fehler: Socket konnte nicht geffnet werden!\n");
		return 1;
	}
	if(connect(client, (struct sockaddr*)&addr, sizeof(addr)) < 0) {
		fprintf(stderr, "+++ Fehler: Verbindung konnte nicht hergestellt werden!\n");
		closesocket(client);
		return 1;
	}
	while(running && !feof(stdin)) {
		if(prompt) {
			sprintf(buffer, "[%li] ", sequence++);
			fprintf(stdout, "%s", buffer);
			fflush(stdout);
		}
		else {
			buffer[0] = '\0';
		}
		if(fgets(&buffer[strlen(buffer)], BUFFER_LENGTH-strlen(buffer), stdin) != NULL) {
			if(echo) {
				fputs(&buffer[0], stdout);
			}
			if(send(client, buffer, strlen(buffer), 0) < 0) {
				fprintf(stderr, "+++ Fehler: Daten konnten nicht gesendet werden!\n");
			}
			if(sscanf(buffer, "[%li]", &request) != 1) {
				request = 0;
			}
			do {
				if(readline(client, buffer, BUFFER_LENGTH) > 0) {
					if(sscanf(buffer, "[%li]", &response) != 1) {
						response = 0;
					}
					fprintf(stdout, "%s", buffer);
				}
				else
					break;
			}	while (request != response);
		}
		else if(ferror(stdin)) {
			perror("+++ Fehler");
		}
	}
	closesocket(client);

    WSACleanup();  /* Cleanup Winsock */
	return 0;
}

/*	-----------  functions  ------------------------------------------------
 */

static void sigterm(int signo)
{
	/*printf("got signal %d\n", signo);*/
	running = 0;
}

static void version(FILE *stream)
{
	fprintf(stream, "%-8s  CANopen over TCP/IP Client, Version %s vom %s\n\n",
					__prog_name,__prog_version,__prog_date);
	fprintf(stream, "\t  Copyright (C) 2009 by UV Software, Friedrichshafen.\n\n");
	fprintf(stream, "-------------------------------------------------------------------------\n");
	fprintf(stream, "Uwe Vogt,  UV Software,  Steinaecker 28,  88048 Friedrichshafen,  Germany\n");
	fprintf(stream, "Fon: +49-7541-6041530, Fax: +49-1803-551809359, Cell fon: +49-170-3801903\n");
	fprintf(stream, "E-Mail: uwe.vogt@uv-software.de, Internet URL: http://www.uv-software.de/\n");
}

static void usage(FILE *stream)
{
	fprintf(stream, "Interfacing CANopen with TCP/IP (Client)\n\n");
	fprintf(stream, "%s <address>:<port> [/PROMPT | /P] [/ECHO | /E]\n",__prog_name);
	fprintf(stream, "%s (/HELP  | /?)\n",__prog_name);
	fprintf(stream, "%s (/ABOUT | /)\n",__prog_name);
}

size_t readline(int fd, char *buf, size_t nbyte)
{
	char chr; size_t pos = 0, res;
	fd_set rdfs;
	struct timeval timeo;
	
    FD_ZERO(&rdfs);
    FD_SET(fd, &rdfs);
    
    timeo.tv_sec  = 66;
    timeo.tv_usec = 0;

	for(;;) {
		if(select(fd+1, &rdfs, NULL, NULL, &timeo) <= 0) {
    		return 0;
		}
		if(FD_ISSET(fd, &rdfs)) {
			if((res = recv(fd, &chr, sizeof(chr), 0)) > 0) {
				if(pos + 1 < nbyte) {
					buf[pos++] = chr;
					buf[pos] = 0;
				}
				if(chr == '\n') {
					return pos;
				}
			}
			else if(res == 0)
				return pos;
			else
				return res;
			if(!running)
				return 0;
		}
	}
}

/*	-------------------------------------------------------------------------
 *	Uwe Vogt,  UV Software,  Steinaecker 28,  88048 Friedrichshafen,  Germany
 *	Fon: +49-7541-6041530, Fax. +49-1803-551809359, Cell fon: +49-170-3801903
 *	E-Mail: uwe.vogt@uv-software.de, Internet URL: http://www.uv-software.de/
 */
